/*
   Create a message port & invert the input we receive from InterPlay

   Copyright 1994 Optonica Limited 1994
   Written by Kevin Stevens

Explanation:
   This is an example of how to receive and modify InterPlay input.

   InterPlay creates a message containing 2 string arguments, the
   first argument being the name of the button pressed, and the
   second being the storage area to put any interpreted string (with a
   maximum of 255 characters + a NULL.

   The message is then sent to a message port created within
   this AREXX script (INPORTNAME).

   This ARexx program then compares the input to all known valid inputs
   and translates them if required and stores any translated string into
   the messages 2nd argument string.

   In order to instruct InterPlay that a translation occured, the reply
   sets the return code to 1.

   Unfortunately this results in some rather nasty looking ARexx code
   that is required to copy ARexx variables into external memory
   locations but it is in fact a fairly simple process

NOTE WELL:
   This code will get called for each input event received
   by InterPlay for any valid button press, and will stop InterPlay
   execution until the message is replied to, so PLEASE be quick and
   handle any longer term operations outside of this process.

Valid strings to send and receive from InterPlay:
   RED,BLUE,GREEN,YELLOW,FORWARD,REVERSE,PLAY,UP,DOWN,LEFT,RIGHT
   correspond to the appropriate buttons on the CD32 hand controller

   IGNORE is a valid RETURN and means ignore this input code
*/

/* global variables used for port names so easy to change */
INPORTNAME = 'FROMIPPORT'
OUTPORTNAME = 'TOIPPORT'

debug = 0      /* enables helpful(!) debuging messages */
if debug > 1 then
   trace r


/* need the support lib to openport() etc */
if ~AddLibrary("rexxsupport.library") then do
   if debug ~= 0 then
      say "can't add rexxsupport.library"
   exit 10
     end

/* ensure there is not already a port of the required name */
if show('P', INPORTNAME) then do
   if debug ~= 0 then
      say "Port " INPORTNAME " already exists"
   exit 10
   end

/* create a port to receive input */
if ~openport(INPORTNAME) then do
   if debug ~= 0 then
      say "can't create port" INPORTNAME
   exit 10
   end

/* main loop - parse input until we get a QUIT instruction */
do until (input = 'QUIT')
   waitpkt(INPORTNAME)
   pkt = getpkt(INPORTNAME)
   if (pkt ~= '0000 0000'x) then do         /* ensure input is valid */
      input = getarg(pkt,0)               /* get the input & translate it */
      select
         when input='UP'      then output='DOWN'
         when input='DOWN'    then output='UP'
         when input='LEFT'    then output='RIGHT'
         when input='RIGHT'   then output='LEFT'
         when input='RED'     then output='BLUE'
         when input='BLUE'    then output='RED'
         when input='QUIT'    then output='BYE'
         when input='GREEN'   then output='YELLOW'
         when input='YELLOW'  then output='GREEN'
         when input='REVERSE' then output='FORWARD'
         when input='FORWARD' then output='REVERSE'
         when input='PLAY'    then output='IGNORE'
         otherwise output=''
         end
      if output ~= '' then do          /* if input translated, inform */
         if debug ~= 0 then
            say input 'translated to' output
         export( import( D2C( C2D(pkt) + 44 ), 4), output, min(length(output)+1, 256), '00'x )
         reply(pkt,1)   /* return 1 = command accepted */
         end
      else do
         if debug ~= 0 then
            say input 'ignored'
         reply(pkt,0)   /* return 0 = command not understood/actioned */
         end
      end      /* end of pkt ~= 0 */
   end      /* end of do .. until */


/* close down & exit cleanly */
closeport INPORTNAME
exit 0



/*
AddLibrary() - ensure a library is available
ARGUMENTS library name, <version> <offset>
RETURNS   1 if available else 0
*/
AddLibrary: PROCEDURE
   if ~show('L',arg(1)) then do
      if ~arg(2, 'E') then
         version = 0
      else
         version = arg(2)

      if ~arg(3, 'E') then
         offset = -30
      else
         offset = arg(3)

      if ~addlib(arg(1),0,offset,version) then
         return 0
      end
   return 1
   end
